from pydantic import (
    BaseModel,
    EmailStr,
    field_validator,
)
from typing import Optional, List
import re


class Owner(BaseModel):
    name: str
    email: EmailStr | None
    additionalEmail: Optional[EmailStr | None]
    phone: Optional[str | None]
    additionalPhone: Optional[str | None]
    comment: Optional[str | None]
    spaces: List[str]

    @field_validator("email")
    def validate_email(cls, value):
        if value:
            if not bool(re.fullmatch(r'[\w.-]+@[\w-]+\.[\w.]+', value)):
                raise ValueError("Email is invalid")
            return value
        else:
            return None



from pydantic import (
    BaseModel,
    Field,
    field_validator,
)
from typing import List
from datetime import datetime
from .status import Status
from .user import User


class Spacetype(BaseModel):
    id: str
    name: str | None = None
    singularName: str | None = None

class Space(BaseModel):
    id: str
    objectId: str
    name: str | None = None
    altName: str | None = None
    type: Spacetype
    status: Status
    indicators: List[str | None] = None
    floor: str | None = None
    plannedArea: float | None = None
    actualArea: float | None = None
    typeDecoration: str | None = None
    areaBTI: float | None = None
    numberBTI: str | None = None
    floorBTI: str | None = None
    externalId: str | None = None
    contractForm: str | None = None
    markupForRegistration: bool = Field(default=True, exclude=True)
    createdBy: User | None = None
    createdAt: int
    modifiedBy: User | None = None
    decorationWarrantyExpiredDate: int | None = None
    constructiveWarrantyExpiredDate: int | None = None
    technicalEquipmentWarrantyExpiredDate: int | None = None

    @field_validator('createdAt', mode='after')
    def convert_timestamps_to_datetime(cls, value):
        if isinstance(value, int):
            return datetime.fromtimestamp(value / 1000)
        return value



from pydantic import (
    BaseModel,
    field_validator,
    Field
)
from typing import List, Optional
from datetime import datetime
from .problem import Problem
from .user import User


class WorkScope(BaseModel):
    value: Optional[str] = None
    unitId: Optional[str] = None
    unitName: Optional[str] = None


class WorkAcceptances(Problem):
    objectId: str
    structureIds: List[str]
    spaceIds: List[str] = []
    acceptanceDate: int
    percent: Optional[float] = 0.0
    comment: Optional[str] = None
    physicalWorkScope: Optional[WorkScope] = None
    planPhysicalWorkScope: Optional[dict] = None
    type: Optional[str] = None
    frontType: Optional[str]
    acceptors: Optional[List[User | str]] = None
    acceptorsActiveGroup: Optional[str] = None
    acceptorsActiveGroupLeader: Optional[User | str] = None
    acceptorsInitialGroup: Optional[str] = None
    acceptorsInitialGroupLeader: Optional[User] = None

    @field_validator('createdAt', 'modifiedAt', 'acceptanceDate', mode='after')
    def convert_timestamps_to_datetime(cls, value):
        if isinstance(value, int):
            return datetime.fromtimestamp(value / 1000)
        return value



import pytest
from tehzor.api import TehzorAPI, TehzorAPIError
from tehzor.models import WorkAcceptances
from config import API_KEY, USER_ID


@pytest.mark.asyncio
async def test_get_work_acceptance_success():
    thz = await TehzorAPI.create(api_key=API_KEY,
                                 user_id=USER_ID
                                 )
    try:
        res = await thz.get_work_acceptance("6655bb0bfc1f87d4ec04d167")
        assert isinstance(res, WorkAcceptances)
    finally:
        await thz.session_close()


@pytest.mark.asyncio
async def test_get_work_acceptance_nonexistent_id():
    thz = await TehzorAPI.create(api_key=API_KEY,
                                 user_id=USER_ID
                                 )
    try:
        with pytest.raises(TehzorAPIError) as e:
            await thz.get_work_acceptance("nonexistent_id")
        assert "Request not found" in str(e.value)
    finally:
        await thz.session_close()



from .attachment import *
from .category import *
from .construction import *
from .health_check import *
from .owner import *
from .problem import *
from .space import *
from .space_meters import *
from .status import *
from .user import *
from .workacceptances import *



from pydantic import (
    BaseModel,
    field_validator,
)
from typing import List, Optional
from datetime import datetime
from .user import User


class SpaceMetersTariff(BaseModel):
    id: Optional[str] = None
    name: Optional[str] = None


class SpaceMeterType(BaseModel):
    id: str
    name: str
    measureUnit: str


class SpaceMetersConsumption(BaseModel):
    id: str | None = None
    value: str | None = None
    tariff: SpaceMetersTariff | None = None
    createdBy: User | None = None
    createdAt: int | None = None
    modifiedBy: User | None = None
    modifiedAt: int | None = None

    @field_validator('createdAt', 'modifiedAt', mode='after')
    def convert_timestamps_to_datetime(cls, value):
        if isinstance(value, int):
            return datetime.fromtimestamp(value / 1000)
        return value


class SpaceMeters(BaseModel):
    id: str
    type: SpaceMeterType | None
    serialNumber: str | None = None
    description: str | None = None
    consumptions: List[SpaceMetersConsumption] | None = None
    createdBy: User | None = None
    createdAt: int | None = None
    modifiedBy: User | None = None
    modifiedAt: int | None = None


from pydantic import (
    BaseModel
)
from typing import Optional


class User(BaseModel):
    id: Optional[str] = None
    fullName: Optional[str] = None
    displayName: Optional[str] = None
    position: Optional[str] = None
    color: Optional[str] = None



from pydantic import (
    BaseModel
)
from typing import Optional


class Attachment(BaseModel):
    id: str
    preview: Optional[dict] = None
    full: Optional[dict] = None
    size: Optional[int] = None




from pydantic import BaseModel
from typing import Optional


class Info(BaseModel):
    status: str
    message: Optional[str] = None


class StatusHealth(BaseModel):
    mongodb: Optional[Info] = None
    redis: Optional[Info] = None


class HealthCheck(BaseModel):
    status: str
    info: StatusHealth
    error: Optional[StatusHealth] = None
    details: StatusHealth



from pydantic import (
    BaseModel,
    field_validator,
    Field
)
from datetime import datetime
from typing import List, Optional
from .attachment import Attachment
from .category import Category
from .construction import Construction
from .status import Status
from .user import User


class ProblemLinks(BaseModel):
    spaceId: Optional[str] = None
    checkId: Optional[str] = None
    internalAcceptanceId: Optional[str] = None
    ownerAcceptanceId: Optional[str] = None
    warrantyClaimId: Optional[str] = None
    checkListId: Optional[str] = None
    checkItemId: Optional[str] = None
    checkRecordId: Optional[str] = None
    taskId: Optional[str] = None
    templateId: Optional[str] = None
    structureId: Optional[str] = None
    workAcceptanceId: Optional[str] = None


class Reason(BaseModel):
    value: Optional[str] = None


class Problem(BaseModel):
    id: str
    object: Optional[Construction] = Field(default=None, exclude=True)
    links: Optional[ProblemLinks] = None
    stage: str
    number: int
    status: Status
    plannedFixDate: Optional[int] = None
    categoryId: str | dict = None
    category: Optional[Category] = Category(id="-", name="Без категории")
    problemTags: Optional[List[str]] = None
    reason: Optional[Reason] = None
    plan: Optional[Status] = None
    floor: Optional[str] = None
    displayLocation: Optional[str] = None
    description: Optional[str] = None
    prescription: Optional[str] = None
    attachments: Optional[List[Attachment]] = None
    performers: Optional[List[User | str]] = Field(default=None, exclude=True)
    watchers: Optional[List[User | str]] = None
    performersActiveGroup: Optional[str] = Field(default=None, exclude=True)
    performersActiveGroupLeader: Optional[User | str] = Field(default=None, exclude=True)
    performersInitialGroup: Optional[str] = Field(default=None, exclude=True)
    performersInitialGroupLeader: Optional[User] = Field(default=None, exclude=True)
    inspectors: Optional[List[Status]] = None
    inspectorsActiveGroup: Optional[str] = None
    critical: Optional[bool] = False
    createdAt: int
    createdBy: Optional[User] = None
    modifiedAt: int
    modifiedBy: Optional[User] = None

    @field_validator('createdAt', 'modifiedAt', mode='after')
    def convert_timestamps_to_datetime(cls, value):
        if isinstance(value, int):
            return datetime.fromtimestamp(value / 1000)
        return value


class ProblemFilter(BaseModel):
    objects: Optional[List[str]] = []  # id строительх объектов
    spaces: Optional[List[str]] = []  # id помещений
